#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpcmd.py
# $Id: wpcmd.py,v 1.4 2009/09/20 20:18:17 elias Alpha1 $
#
# The WordPress Shell
# Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
wpcmd.py

The command line driven WordPress shell.

Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
"""

# Important to do this before any other things are done...
import locale
locale.setlocale(locale.LC_ALL, '')

import sys
import optparse

import wpsh.commands as cmd
from wpsh.wpshbase import VERSION, LICENSE, DONATE, WARRANTY



ID = '$Id: wpcmd.py,v 1.4 2009/09/20 20:18:17 elias Alpha1 $'



def main():
    "Main function."
    p = optparse.OptionParser(usage='%prog [options] filelist',
                              version='%prog ' + VERSION)
    p.add_option('-v', '--verbose',
                 action='store_true',
                 dest='verbose',
                 help='print messages while processing',
                 default=False)
    # Configuration options
    p.add_option('-L', '--bloglist',
                 action='store_true',
                 dest='listblogs',
                 help='show the names of the configured blogs', 
                 default=False)
    p.add_option('-A', '--blogadd', 
                 action='store_true',
                 dest='addblog',
                 help='validate a blog and add it to the configuration',
                 default=False)
    p.add_option('-R', '--blogremove',
                 metavar='BLOGNAME',
                 action='store',
                 dest='removeblog',
                 help='remove a blog from the configuration',
                 default=False)
    p.add_option('-I', '--bloginfo',
                 metavar='BLOGNAME',
                 action='store',
                 dest='bloginfo',
                 help='show some blog informations',
                 default=False)
    p.add_option('-G', '--blogrevalidate',
                 metavar='BLOGNAME',
                 action='store',
                 dest='revalidateblog',
                 help='revalidate an already configured blog',
                 default=False)
    p.add_option('-K', '--blogcategorytree',
                 metavar='BLOGNAME',
                 action='store',
                 dest='blogcategorytree',
                 help='print the category tree for the blog',
                 default=False)
    p.add_option('-T', '--blogposttemplate',
                 metavar='BLOGPATTERN',
                 action='store',
                 dest='blogposttemplate',
                 help='create post templates for blogs matching pattern',
                 default=False)
    # Other funny stuff
    p.add_option('-V', '--blogvisit',
                 metavar='BLOGNAME',
                 action='store',
                 dest='visitblog',
                 help='visit the blog with your webbrowser',
                 default=False)
    # The legal stuff and other informations
    p.add_option('-C', '--copyleft',
                 action='store_true',
                 dest='copyleft',
                 help='read the license information for the program',
                 default=False)
    p.add_option('-W', '--warranty',
                 action='store_true',
                 dest='warranty',
                 help='read about the (no-)warranty for the program',
                 default=False)
    p.add_option('-S', '--support',
                 action='store_true',
                 dest='support',
                 help='read how to support me',
                 default=False)
    try:
        (opt, a) = p.parse_args()
    except optparse.OptParseError:
        p.print_help()
        exit('Illegal option')
    # General options
    cmd.verbose = opt.verbose
    if opt.copyleft:
        print LICENSE
        sys.exit()
    if opt.warranty:
        print WARRANTY
        sys.exit()
    if opt.support:
        print DONATE
        sys.exit()
    # Blog configuration options
    if opt.listblogs:
        cmd.listBlogs()
    elif opt.addblog:
        # The parameters may be given at command line, especally from
        # lazy ones like me...
        while len(a) < 5:
            a.append(None)
        cmd.addBlog(a[0], a[1], a[2], a[3], a[4])
    elif opt.removeblog:
        cmd.removeBlog(opt.removeblog)
    elif opt.visitblog:
        cmd.visitBlog(opt.visitblog)
    elif opt.bloginfo:
        cmd.blogInformation(opt.bloginfo)
    elif opt.revalidateblog:
        cmd.revalidateBlog(opt.revalidateblog)
    elif opt.blogcategorytree:
        cmd.categoryTree(opt.blogcategorytree)
    elif opt.blogposttemplate:
        cmd.blogPostTemplate(opt.blogposttemplate)
    else:
        if len(a) > 0:
            for file in a:
                cmd.postFile(file)
        else:
            cmd.interactiveMode()



if __name__ == '__main__':
    main()



########################################################################
#
# $Log: wpcmd.py,v $
# Revision 1.4  2009/09/20 20:18:17  elias
# Source cleanup for alpha release.
#
# Revision 1.3  2009/09/20 02:32:20  elias
# Post functionality.
# Upload functionality.
# Possibility to blog from stdin.
#
# Revision 1.2  2009/09/18 17:01:31  elias
# Intermediate state.
#
# Revision 1.1  2009/09/18 13:34:10  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
