#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpcmd/wpshell.py
# $Id: wpshell.py,v 1.5 2009/09/20 20:12:11 elias Alpha1 $
#
# The WordPress Shell
# Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################



import os.path
try:
    # Use that funny readline stuff, if available on the system.
    # Of course with a history... ;-)
    import readline
    # The history mechanism is commented out for security reasons.
    # If someone adds a blog in the shell with config add, the 
    # username and password are kept in the history. In later versions,
    # a better password input is required.
    #
    #history_file = os.path.expanduser('~/.wpcmdhist')
    #try:
    #    readline.read_history_file(history_file)
    #except IOError:
    #    pass
    #import atexit
    #atexit.register(readline.write_history_file, history_file)
    #readline.set_history_length(500) # more than enough!
except:
    pass
import cmd


import commands as wp
import wpshbase



ID = '$Id: wpshell.py,v 1.5 2009/09/20 20:12:11 elias Alpha1 $'



class TooManyArguments(wpshbase.Error):
    """Too many arguments for a command"""
    pass



class MissingArgument(wpshbase.Error):
    """Argument missing"""
    pass



class NoBlogUsed(wpshbase.Error):
    """No blog in use"""
    pass



class NoAdminPrivileges(wpshbase.Error):
    """No administrative privileges"""
    pass



class UnknownAction(wpshbase.Error):
    """Unknown action for a command"""
    pass



class WPShell(cmd.Cmd, wpshbase.Base):
    """\
    The interactive command line for wordpress administration.
    
    Funny, nerdy stuff!

    This is a user interface to the functions in commands.py, but it
    is intended to be a useful and comfortable one. 
    """

    prompt = '> '
    intro = """\
*** wpcmd (version %s) ***
Copyright (c) 2009 Elias Schwerdtfeger, http://www.tamagothi.de/

Wpcmd Publishes Content Mouseless Directly  ;-)

Wpcmd is FREE SOFTWARE, and you are welcome to distribute it or do your 
own derivative work on it. Type 'info copying' for more information. 

There is NO WARRANTY for wpcmd, type 'info warranty' for more information.

If you want to support me, type 'info support' for more information.

For online help type '?' or 'help'.
""" % (wpshbase.VERSION)

    def __init__(self):
        """Constructor."""
        cmd.Cmd.__init__(self)
        self.bc = None
        self.changePrompt()

    # helper methods

    def changePrompt(self):
        """The prompt shows the blog in use."""
        if self.bc:
            self.prompt = 'wpcmd@%s> ' % (self.bc.name)
        else:
            self.prompt = 'wpcmd> '

    def yn(self, prompt, *args):
        """Simple yes or no question."""
        p = prompt % args
        while True:
            yn = wp.uinput('%s (y/n)? ', p).strip()
            fyn = yn[0].lower()
            if fyn == 'y' or fyn == 'j' or fyn == '1':
                return True
            elif fyn == 'n' or fyn == '0':
                return False

    def getBc(self):
        """Get the current blog with error checking."""
        if not self.bc:
            raise NoBlogUsed, 'no blog in use'
        return self.bc

    def getAdminBc(self):
        bc = self.getBc()
        if not bc.admin:
            raise NoAdminPrivileges, 'you cannot administrate'
        return bc

    def exc(self, line, *args):
        """Handle an exception."""
        l = line % args
        wp.exchandler(l)

    # My own cmdloop catches unhandled exceptions without exiting.

    def cmdloop(self):
        """Command dispatcher loop.

        This one catches otherwise unhandled exceptions without exiting
        the shell, which is a little more user-friendly. But it is a
        little dangerous too, it can lead in infinit exception loops 
        which forces the poor user to kill the process. 
        """
        done = False
        firsttime = True
        while not done:
            try:
                if firsttime:
                    firsttime = False
                    cmd.Cmd.cmdloop(self)
                else:
                    cmd.Cmd.cmdloop(self, 'Command loop restarted')
                done = True
            except KeyboardInterrupt:
                # Ctrl-C is always possible
                done = True
            except:
                # And this is for the flying pigs
                self.exc('weird unhandled runtime error')

    # command line completion helpers

    def generalCompleter(self, txt, list):
        """Try to complete the text with a list."""
        rlist = []
        for i in list:
            if i.startswith(txt):
                rlist.append(i)
        return rlist

    def blognameCompleter(self, txt):
        """Complete a blogname."""
        return self.generalCompleter(txt, wp.theConfig.getBlogList())

    ####################################################################
    #
    # Command implementation
    #
    ####################################################################

    # Little commands
    # ---------------

    def help_help(self):
        print "Read some help on the commands."

    def do_EOF(self, line):
        """The wpcmd shell quits on end of file."""
        print
        return True

    def do_quit(self, line):
        """Quit the wpcmd shell."""
        return True

    # Visit a blog
    # ------------

    def do_visit(self, line):
        """Visit a blog with your web browser."""
        wp.visitBlog(line.strip())

    def complete_visit(self, text, line, begidx, endidx):
        n = line.count(' ')
        if n == 1:
            return self.blognameCompleter(text)
        else:
            return []

    # Info command
    # ------------

    def help_info(self):
        print """\
info - Display some general informations about the program

Usage: show [action]

Actions:
info license  - Show information on the program's license
info warranty - Show information on the (no-)warranty
info support  - Show how to support my and the program's further development
"""

    def complete_info(self, text, line, begidx, endidx):
        n = line.count(' ')
        if n == 1:
            return self.generalCompleter(text, ['license', 'warranty',
                                                'support'])
        else:
            return []

    def do_info(self, line):
        try:
            action = line.strip()
            if not action:
                raise MissingArgument, 'no action given'
            if action == 'license':
                print wpshbase.LICENSE
            elif action == 'warranty':
                print wpshbase.WARRANTY
            elif action == 'support':
                print wpshbase.DONATE
            else:
                raise UnknownAction, 'Undefined action "%s"' % (action,)
        except:
            self.exc('failed to show information')

    def do_list(self, line):
        """A shorthand for config list, it lists all configured blogs."""
        # This may not fail, no exception handling
        wp.listBlogs()

    # Config command
    # --------------

    def help_config(self):
        print """\
config - Administration of the configured blogs in wpcmd

Usage: config [action] [blogname]

Actions:
config list                        - List the configured blognames
config add [blogname]              - Add a blog to the configuration
config remove [blogname]           - Remove a blog from the configuration
config info [blogname]             - Show information on a configured blog
config validate [blogname]         - Revalidates a blog
config categories [blogname]       - Configures the categories
config rename [blogname] [newname] - Renames a configured blog
config comment [blogname] [text]   - Changes the comment for a blog
config posttemplate [blogpattern]  - Create post templates for blogs
"""

    def complete_config(self, text, line, begidx, endidx):
        n = line.count(' ')
        if n == 1:
            return self.generalCompleter(text, 
                                          ['list', 'add', 'remove',
                                           'info', 'validate', 'categories',
                                           'rename', 'comment', 
                                           'posttemplate'])
        elif n == 2:
            return self.blognameCompleter(text)
        else:
            return []

    def do_config(self, line):
        p = line.split()
        bname = None
        try:
            if len(p) == 0:
                raise MissingArgument, 'no action specified'
            if len(p) >= 2:
                bname = p[1]
            action = p[0]
            # list may be called without blog name
            if action == 'list':
                if bname:
                    wp.blogInformation(bname)
                else:
                    wp.listBlogs()
            # rename needs a third argument
            elif action == 'rename':
                if len(p) < 3:
                    raise MissingArgument, 'not enough arguments'
                elif len(p) > 3:
                    raise TooManyArguments, 'to many arguments'
                else:
                    flag = False
                    if self.bc and bname == self.bc.name:
                        flag = True
                    wp.blogRename(bname, p[2])
                    if flag:
                        self.bc = wp.theConfig.getBlogConfig(p[2])
                        self.changePrompt()
            # comment takes the rest of line as comment tokens
            elif action == 'comment':
                if len(p) < 3:
                    raise MissingArgument, 'no comment given'
                comment = ' '.join(p[2:])
                wp.blogComment(bname, comment)
            else:
                if len(p) > 2:
                    raise TooManyArguments, 'too many arguments'
                # all other actions require a blog name
                if not bname:
                    raise MissingArgument, 'no blog name given'
                elif action == 'posttemplate':
                    wp.blogPostTemplate(bname)
                elif action == 'add':
                    wp.addBlog(bname)
                elif action == 'remove':
                    if self.yn('Really REMOVE %s', bname):
                        if self.bc and bname == self.bc.name:
                            self.bc = None
                            self.changePrompt()
                        wp.removeBlog(bname)
                elif action == 'info':
                    wp.blogInformation(bname)
                elif action == 'validate':
                    wp.revalidateBlog(bname)
                elif action == 'categories':
                    wp.blogReadCategories(bname)
                else:
                    raise UnknownAction, 'illegal action "%s"' % (action,)
        except:
            self.exc('config command failed')

    # Category administration
    # -----------------------

    def complete_category(self, text, line, begidx, endidx):
        s = line.count(' ')
        if not self.bc:
            return []
        elif s <= 1:
            return self.generalCompleter(text, ['tree', 'add', 'visit',
                                                'remove'])
        else:
            # Buggy!
            txt = line[begidx:endidx]
            return self.generalCompleter(txt, self.bc.cats.keys())

    def do_category(self, line):
        try:
            bc = self.getBc()
            if len(line) == 0:
                raise MissingArgument, 'no action specified'
            p = line.split(None, 1)
            action = p[0]
            # The category tree is the simplest thing
            if action == 'tree':
                if len(p) > 1:
                    raise TooManyArguments, 'no arguments allowed'
                wp.categoryTree(bc.name)
                return
            # Visiting the page for a category is simple too
            if action == 'visit':
                if len(p) < 2:
                    raise MissingArgument, 'no category name given'
                wp.visitCategory(bc, p[1])
                return
            # For all other actions I wanna see admin privileges
            if not bc.admin:
                raise NoAdminPrivileges, 'administrative privileges required'
            if action == 'add':
                if len(p) == 1:
                    wp.addCategory(bc, None)
                elif len(p) == 2:
                    wp.addCategory(bc, p[1])
                else:
                    raise TooManyArguments, 'only a parent category allowed'
                return
            # And now a category name is always needed
            if len(p) < 2:
                raise MissingArgument, 'no category name given'
            catname = p[1]
            if action == 'remove':
                wp.removeCategory(bc, catname)
                return
            raise UnknownAction, 'illegal action "%s"' % (action,)
        except:
            self.exc('category command failed')

    def help_category(self):
        print "category - Work with the categories of the blog in use"
        if not self.bc:
            print "You must select a blog with 'use' first."
        print """\

Usage: category [action] parameters

Actions:
category tree          List the category tree
category add [parent]  Add a category (parent optional)
category visit [name]  Visit a category with the web browser
category remove [name] Remove a category
"""
        pass

    # Use blog and leave blog
    # -----------------------

    def do_use(self, line):
        """select a blog for administrative commands"""
        try:
            line = line.strip()
            if len(line) == 0:
                raise MissingArgument, 'no blog specified'
            self.bc = wp.theConfig.getBlogConfig(line.strip())
            if not self.bc.isValid():
                raise NoBlogUsed, '%s is invalid' % (bc.name,)
        except:
            self.bc = None
            self.exc('Failed to use blog %s', line)
        self.changePrompt()
    
    def complete_use(self, text, line, begidx, endidx):
        if line.count(' ') == 1:
            return self.blognameCompleter(text)
        else:
            return []

    def do_leave(self, line):
        """leave the blog you currently use"""
        try:
            if self.bc == None:
                raise NoBlogUsed, 'You do not use a blog'
            self.bc = None
            self.changePrompt()
        except:
            self.exc('Failed to leave blog')

        

########################################################################
#
# $Log: wpshell.py,v $
# Revision 1.5  2009/09/20 20:12:11  elias
# Remove history file mechanism for alpha release.
#
# Revision 1.4  2009/09/20 04:05:38  elias
# First alpha release state.
#
# Revision 1.3  2009/09/18 22:09:03  elias
# Intermediate state.
# Complete local blog administration.
#
# Revision 1.2  2009/09/18 15:32:39  elias
# Intermediate saving, WIP.
# Changed commandline syntax to keyword action parameter.
# Started commandline auto-complete for readline-enabled systems.
# Started to use the exception handler as a method.
#
# Revision 1.1  2009/09/17 17:15:50  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
